@@ -46,76 +46,57 @@ impl Zig {
         let is_windows_gnu = target
             .map(|x| x.contains("windows-gnu"))
             .unwrap_or_default();
-        // Replace libgcc_s with libunwind
-        let cmd_args: Vec<String> = cmd_args
-            .iter()
-            .map(|arg| {
-                let arg = if arg == "-lgcc_s" {
-                    "-lunwind".to_string()
-                } else if is_windows_gnu && arg == "-lgcc_eh" {
-                    "-lc++".to_string()
-                } else if arg.starts_with('@') && arg.ends_with("linker-arguments") {
-                    // rustc passes arguments to linker via an @-file when arguments are too long
-                    // See https://github.com/rust-lang/rust/issues/41190
-                    let content = fs::read(arg.trim_start_matches('@'))?;
-                    let link_args: Vec<_> = str::from_utf8(&content)?
-                        .split('\n')
-                        .filter_map(|arg| {
-                            if arg == "-lgcc_s" {
-                                return Some("-lunwind");
-                            } else if is_windows_gnu && arg == "-lgcc_eh" {
-                                return Some("-lc++");
-                            }
-                            if is_musl {
-                                if arg.ends_with(".o")
-                                    && arg.contains("self-contained")
-                                    && arg.contains("crt")
-                                {
-                                    return None;
-                                }
-                                if arg.ends_with(".rlib") && arg.contains("liblibc-") {
-                                    return None;
-                                }
-                            }
-                            if is_windows_gnu && (arg == "-l:libpthread.a" || arg == "-lgcc") {
-                                return None;
-                            }
-                            Some(arg)
-                        })
-                        .collect();
-                    fs::write(arg.trim_start_matches('@'), link_args.join("\n").as_bytes())?;
-                    arg.to_string()
-                } else {
-                    arg.to_string()
-                };
-                Ok(arg)
-            })
-            .filter(|arg| {
-                // Avoids duplicated symbols with both zig musl libc and the libc crate
-                if let Ok(arg) = arg.as_ref() {
-                    if is_musl {
-                        if arg.ends_with(".o")
-                            && arg.contains("self-contained")
-                            && arg.contains("crt")
-                        {
-                            return false;
-                        }
-                        if arg.ends_with(".rlib") && arg.contains("liblibc-") {
-                            return false;
-                        }
-                    }
-                    if is_windows_gnu && (arg == "-l:libpthread.a" || arg == "-lgcc") {
-                        return false;
-                    }
+
+        let filter_link_arg = |arg: &str| {
+            if arg == "-lgcc_s" {
+                // Replace libgcc_s with libunwind
+                return Some("-lunwind".to_string());
+            }
+            if is_windows_gnu {
+                if arg == "-lgcc_eh" {
+                    // zig doesn't provide gcc_eh alternative
+                    // We use libc++ to replace it on windows gnu targets
+                    return Some("-lc++".to_string());
+                } else if arg == "-lwindows" || arg == "-l:libpthread.a" || arg == "-lgcc" {
+                    return None;
                 }
-                true
-            })
-            .collect::<Result<_>>()?;
+            }
+            if is_musl {
+                // Avoids duplicated symbols with both zig musl libc and the libc crate
+                if arg.ends_with(".o") && arg.contains("self-contained") && arg.contains("crt") {
+                    return None;
+                }
+                if arg.ends_with(".rlib") && arg.contains("liblibc-") {
+                    return None;
+                }
+            }
+            Some(arg.to_string())
+        };
+
+        let mut new_cmd_args = Vec::with_capacity(cmd_args.len());
+        for arg in cmd_args {
+            let arg = if arg.starts_with('@') && arg.ends_with("linker-arguments") {
+                // rustc passes arguments to linker via an @-file when arguments are too long
+                // See https://github.com/rust-lang/rust/issues/41190
+                let content = fs::read(arg.trim_start_matches('@'))?;
+                let link_args: Vec<_> = str::from_utf8(&content)?
+                    .split('\n')
+                    .filter_map(filter_link_arg)
+                    .collect();
+                fs::write(arg.trim_start_matches('@'), link_args.join("\n").as_bytes())?;
+                Some(arg.to_string())
+            } else {
+                filter_link_arg(arg)
+            };
+            if let Some(arg) = arg {
+                new_cmd_args.push(arg);
+            }
+        }
         let (zig, zig_args) = Self::find_zig()?;
         let mut child = Command::new(zig)
             .args(zig_args)
             .arg(cmd)
-            .args(cmd_args)
+            .args(new_cmd_args)
             .spawn()
             .with_context(|| format!("Failed to run `zig {}`", cmd))?;
         let status = child.wait().expect("Failed to wait on zig child process");
@@ -170,6 +151,8 @@ impl Zig {
     }
 }
 
+/// Prepare wrapper scripts for `zig cc` and `zig c++` and returns their paths
+///
 /// We want to use `zig cc` as linker and c compiler. We want to call `python -m ziglang cc`, but
 /// cargo only accepts a path to an executable as linker, so we add a wrapper script. We then also
 /// use the wrapper script to pass arguments and substitute an unsupported argument.
@@ -201,17 +184,18 @@ pub fn prepare_zig_linker(target: &str) -> Result<(PathBuf, PathBuf)> {
     let file_ext = if cfg!(windows) { "bat" } else { "sh" };
     let zig_cc = format!("zigcc-{}.{}", target, file_ext);
     let zig_cxx = format!("zigcxx-{}.{}", target, file_ext);
+    let cc_args = "-g"; // prevent stripping
     let cc_args = match triple.operating_system {
         OperatingSystem::Linux => format!(
-            "-target {}-linux-{}{}",
-            arch, triple.environment, abi_suffix
+            "-target {}-linux-{}{} {}",
+            arch, triple.environment, abi_suffix, cc_args,
         ),
         OperatingSystem::MacOSX { .. } | OperatingSystem::Darwin => {
-            format!("-target {}-macos-gnu{}", arch, abi_suffix)
+            format!("-target {}-macos-gnu{} {}", arch, abi_suffix, cc_args)
         }
         OperatingSystem::Windows { .. } => format!(
-            "-target {}-windows-{}{}",
-            arch, triple.environment, abi_suffix
+            "-target {}-windows-{}{} {}",
+            arch, triple.environment, abi_suffix, cc_args,
         ),
         _ => bail!("unsupported target"),
     };
@@ -231,6 +215,7 @@ pub fn prepare_zig_linker(target: &str) -> Result<(PathBuf, PathBuf)> {
     Ok((zig_cc, zig_cxx))
 }
 
+/// Write a zig cc wrapper batch script for unix
 #[cfg(target_family = "unix")]
 fn write_linker_wrapper(path: &Path, command: &str, args: &str) -> Result<()> {
     let mut custom_linker_file = OpenOptions::new()
